"""
Classifies: CHEBI:28966 chlorophyll
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_chlorophyll(smiles: str):
    """
    Determines if a molecule is a chlorophyll based on the presence of a fifth ring
    beyond the four pyrrole-like rings, and the presence of a magnesium atom.
    The function also checks for the presence of a long phytol chain.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a chlorophyll, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Count the number of rings
    ring_info = mol.GetRingInfo()
    num_rings = len(ring_info.AtomRings())

    if num_rings < 5:
        return False, "Less than 5 rings found"

    # Check for the presence of a magnesium atom
    has_mg = any(atom.GetSymbol() == 'Mg' for atom in mol.GetAtoms())
    if not has_mg:
        return False, "No magnesium atom found"

    # Check for the presence of a phytol chain
    phytol_pattern = Chem.MolFromSmarts('C=CC(C)(CCC=C(C)C)')
    has_phytol = mol.HasSubstructMatch(phytol_pattern)

    if has_phytol:
        return True, "Chlorophyll with phytol chain"
    else:
        return True, "Chlorophyll without phytol chain"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:28966',
                          'name': 'chlorophyll',
                          'definition': 'A family of magnesium porphyrins, '
                                        'defined by the presence of a fifth '
                                        'ring beyond the four pyrrole-like '
                                        'rings. The rings can have various '
                                        'side chains which usually include a '
                                        'long phytol chain.',
                          'parents': ['CHEBI:25111']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 4,
    'num_false_positives': 62,
    'num_true_negatives': 183832,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.06060606060606061,
    'recall': 1.0,
    'f1': 0.1142857142857143,
    'accuracy': 0.9996628565835409}


"""
Classifies: CHEBI:139588 alpha-hydroxy ketone
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_alpha_hydroxy_ketone(smiles: str):
    """
    Determines if a molecule contains an alpha-hydroxy ketone group.
    An alpha-hydroxy ketone has a hydroxy group adjacent to a ketone.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule contains alpha-hydroxy ketone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # SMARTS pattern for alpha-hydroxy ketone
    # [OH1] matches hydroxyl group
    # [CH,CH2] matches carbon with 1 or 2 hydrogens
    # C(=O) matches ketone carbonyl
    pattern = Chem.MolFromSmarts('[OH1]-[CH,CH2]-C(=O)')
    
    # Alternative pattern for alpha-hydroxy ketone where carbon has other substituents
    pattern2 = Chem.MolFromSmarts('[OH1]-[C]-C(=O)')

    matches = mol.GetSubstructMatches(pattern)
    matches2 = mol.GetSubstructMatches(pattern2)
    
    all_matches = matches + matches2
    
    if len(all_matches) > 0:
        return True, f"Contains {len(all_matches)} alpha-hydroxy ketone group(s)"
    else:
        return False, "No alpha-hydroxy ketone group found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:139588',
                          'name': 'alpha-hydroxy ketone',
                          'definition': 'An alpha-oxyketone that has a hydroxy '
                                        'group as the alpha-oxy moiety.',
                          'parents': ['CHEBI:30879', 'CHEBI:52396']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 50,
    'num_false_positives': 100,
    'num_true_negatives': 1210,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.3333333333333333,
    'recall': 0.9803921568627451,
    'f1': 0.4975124378109452,
    'accuracy': 0.925789860396767}


"""
Classifies: CHEBI:132157 hydroxy-1,4-naphthoquinone
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_hydroxy_1_4_naphthoquinone(smiles: str):
    """
    Determines if a molecule is a hydroxy-1,4-naphthoquinone.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a hydroxy-1,4-naphthoquinone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # SMARTS pattern for 1,4-naphthoquinone core
    # Two carbonyl groups in para position on a naphthalene ring
    naphthoquinone_pattern = Chem.MolFromSmarts('O=C1C=CC(=O)c2ccccc12')
    
    # SMARTS pattern for hydroxy group
    hydroxy_pattern = Chem.MolFromSmarts('[OH]')

    if naphthoquinone_pattern is None or hydroxy_pattern is None:
        return None, "Invalid SMARTS pattern"

    # Check for 1,4-naphthoquinone core
    if not mol.HasSubstructMatch(naphthoquinone_pattern):
        return False, "No 1,4-naphthoquinone core structure found"

    # Check for at least one hydroxy group
    if not mol.HasSubstructMatch(hydroxy_pattern):
        return False, "No hydroxy group found"

    # Count hydroxy groups and determine their positions
    hydroxy_matches = len(mol.GetSubstructMatches(hydroxy_pattern))
    
    # Get the positions of hydroxy groups
    atom_indices = mol.GetSubstructMatches(hydroxy_pattern)
    positions = []
    for match in atom_indices:
        oh_atom = mol.GetAtomWithIdx(match[0])
        for neighbor in oh_atom.GetNeighbors():
            if neighbor.GetIsAromatic():
                positions.append("aromatic ring")
            else:
                positions.append("aliphatic")

    position_str = ", ".join(set(positions))
    return True, f"Hydroxy-1,4-naphthoquinone with {hydroxy_matches} hydroxy group(s) on {position_str}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:132157',
                          'name': 'hydroxy-1,4-naphthoquinone',
                          'definition': 'Any member of the class of '
                                        '1,4-naphthoquinones in which the '
                                        'naphthoquinone moiety is substituted '
                                        'by at least one hydroxy group.',
                          'parents': ['CHEBI:132142', 'CHEBI:132155']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': 'Attempt failed: Python argument types in\n'
               '    Mol.HasSubstructMatch(Mol, NoneType)\n'
               'did not match C++ signature:\n'
               '    HasSubstructMatch(RDKit::ROMol self, RDKit::MolBundle '
               'query, RDKit::SubstructMatchParameters params=True)\n'
               '    HasSubstructMatch(RDKit::ROMol self, RDKit::ROMol query, '
               'RDKit::SubstructMatchParameters params)\n'
               '    HasSubstructMatch(RDKit::ROMol self, RDKit::MolBundle '
               'query, bool recursionPossible=True, bool useChirality=False, '
               'bool useQueryQueryMatches=False)\n'
               '    HasSubstructMatch(RDKit::ROMol self, RDKit::ROMol query, '
               'bool recursionPossible=True, bool useChirality=False, bool '
               'useQueryQueryMatches=False)',
    'attempt': 2,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 100,
    'num_true_negatives': 38660,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.02912621359223301,
    'recall': 1.0,
    'f1': 0.05660377358490566,
    'accuracy': 0.9974202203131852}


"""
Classifies: CHEBI:24654 hydroxy fatty acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.rdMolDescriptors import CalcMolFormula

def is_hydroxy_fatty_acid(smiles: str):
    """
    Determines if a molecule is a hydroxy fatty acid, defined as a fatty acid carrying one or more hydroxy substituents.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a hydroxy fatty acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
    
    # Check for carboxylic acid group
    carboxylic_pattern = Chem.MolFromSmarts('C(=O)[OH]')
    carboxylic_pattern_ionic = Chem.MolFromSmarts('C(=O)[O-]')
    if not (mol.HasSubstructMatch(carboxylic_pattern) or mol.HasSubstructMatch(carboxylic_pattern_ionic)):
        return False, "No carboxylic acid group found"

    # Check for hydroxy groups
    hydroxy_pattern = Chem.MolFromSmarts('[OH]')
    hydroxy_matches = mol.GetSubstructMatches(hydroxy_pattern)
    
    # Get carboxylic acid OH indices
    carboxylic_matches = mol.GetSubstructMatches(carboxylic_pattern)
    carboxylic_o_indices = [match[1] for match in carboxylic_matches]
    
    # Count hydroxy groups that are not part of carboxylic acid
    hydroxy_count = 0
    for match in hydroxy_matches:
        if match[0] not in carboxylic_o_indices:
            hydroxy_count += 1
            
    if hydroxy_count == 0:
        return False, "No hydroxy substituents found"

    # Check for too many rings (fatty acids typically have few or no rings)
    ring_info = mol.GetRingInfo()
    ring_count = ring_info.NumRings()
    if ring_count > 3:  # Increased tolerance for rings
        return False, "Too many rings for a fatty acid"

    # Calculate carbon chain characteristics
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if carbon_count < 3:  # Reduced minimum carbon requirement
        return False, "Carbon chain too short for a fatty acid"

    # Check for aromatic systems (fatty acids shouldn't be aromatic)
    aromatic_atoms = sum(1 for atom in mol.GetAtoms() if atom.GetIsAromatic())
    if aromatic_atoms > 2:  # Allow some aromatic atoms
        return False, "Contains too many aromatic atoms"

    # Check for peptide bonds
    peptide_pattern = Chem.MolFromSmarts('[NX3][CX3](=[OX1])[#6]')
    if mol.HasSubstructMatch(peptide_pattern):
        return False, "Contains peptide bond"

    # Calculate ratio of carbons to other heavy atoms (excluding H)
    other_heavy_atoms = sum(1 for atom in mol.GetAtoms() 
                           if atom.GetSymbol() not in ['C', 'H'])
    if other_heavy_atoms > carbon_count * 0.75:  # Increased tolerance for heteroatoms
        return False, "Too many heteroatoms for a fatty acid"

    return True, f"Hydroxy fatty acid with {hydroxy_count} hydroxy group(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24654',
                          'name': 'hydroxy fatty acid',
                          'definition': 'Any fatty acid carrying one or more '
                                        'hydroxy substituents.',
                          'parents': ['CHEBI:35366', 'CHEBI:35868']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.5172413793103448 is too low.\n'
               "True positives: [('CCCCCCCCCCCCCCC(O)C(O)=O', 'Hydroxy fatty "
               "acid with 2 hydroxy group(s)'), "
               "('OCCCCCCCCCCCCCCCCCC[C@@H](O)CC(O)=O', 'Hydroxy fatty acid "
               "with 3 hydroxy group(s)'), ('C[C@@H](O)CC\\\\C=C\\\\C(O)=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('CC(C)CCCC(C)CCCC(C)CCCC(C)C(O)C(O)=O', 'Hydroxy fatty acid "
               "with 2 hydroxy group(s)'), "
               "('OCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC(O)=O', 'Hydroxy fatty "
               "acid with 2 hydroxy group(s)'), "
               "('CCCCCCC(O)C\\\\C=C/CCCCCCCC(O)=O', 'Hydroxy fatty acid with "
               "2 hydroxy group(s)'), ('C[C@@H](O)CCCCCCC[C@@H](O)CC(O)=O', "
               "'Hydroxy fatty acid with 3 hydroxy group(s)'), "
               "('C(CCCCCCCC)CCCCCCC[C@@H](C(O)=O)O', 'Hydroxy fatty acid with "
               "2 hydroxy group(s)'), ('C[C@@H](O)CC[C@H](CC(O)=O)C(C)=C', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('[H][C@@]1(C\\\\C=C/CCCCC)O[C@@]1([H])\\\\C=C\\\\C(O)C\\\\C=C/CCCC(O)=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('C[C@H](O)C#CC#CC#CC#CCCCCCCCC(O)=O', 'Hydroxy fatty acid "
               "with 2 hydroxy group(s)'), "
               "('CCCCC\\\\C=C/C=C/[C@H](O)CCCCCCCC(O)=O', 'Hydroxy fatty acid "
               "with 2 hydroxy group(s)'), "
               "('O=C(CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\[C@H](CCCC)O)O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('O[C@H](CCCC(O)=O)CC\\\\C=C\\\\C=C\\\\[C@H](O)C/C=C\\\\CCCCC', "
               "'Hydroxy fatty acid with 3 hydroxy group(s)'), "
               "('CCCCC(O)CCCCCCCCC(O)=O', 'Hydroxy fatty acid with 2 hydroxy "
               "group(s)'), "
               "('OC(CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C=C\\\\C(C/C=C\\\\CC)O)=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('C(=C\\\\C/C=C\\\\CCCC(C)O)\\\\CCCCCCCC(=O)O', 'Hydroxy fatty "
               "acid with 2 hydroxy group(s)'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCCC[C@@H](O)C(O)=O', 'Hydroxy "
               "fatty acid with 2 hydroxy group(s)'), "
               "('C(/C=C/C=C/[C@H]([C@@H](O)C/C=C\\\\C/C=C\\\\CC)O)=C/C/C=C\\\\CCC(O)=O', "
               "'Hydroxy fatty acid with 3 hydroxy group(s)'), "
               "('O[C@@H](CCCCC)/C=C/C=C\\\\C/C=C\\\\C=C\\\\C(=O)CCCC(O)=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('CCCCCCC(O)C(O)\\\\C=C\\\\C(O)CCCCCCC(O)=O', 'Hydroxy fatty "
               "acid with 4 hydroxy group(s)'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C[C@H](O)\\\\C=C\\\\C=C/CCCC(O)=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('O[C@@H]([C@@H](O)C(O)/C=C\\\\C/C=C\\\\CCCC(O)=O)C/C=C\\\\CCCCC', "
               "'Hydroxy fatty acid with 4 hydroxy group(s)'), "
               "('OCCCCCCCCCCCCCCCCCCCCCCCC(O)=O', 'Hydroxy fatty acid with 2 "
               "hydroxy group(s)'), "
               "('OC(CCCC(O)=O)\\\\C=C\\\\C=C\\\\C=C\\\\C(O)C\\\\C=C/C=C\\\\C(O)CC', "
               "'Hydroxy fatty acid with 4 hydroxy group(s)'), "
               "('C(CCCCCCC/C=C\\\\C=C\\\\[C@H](CCCCC)O)(=O)O', 'Hydroxy fatty "
               "acid with 2 hydroxy group(s)'), "
               "('C(CCCCCCCC(=O)OC(CCCCCCCCCCC(=O)O)CCCCCC)CCCCCCCCC', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('C(\\\\C([C@H]1[C@H](C/C=C\\\\CCCCC)O1)O)=C\\\\C/C=C\\\\CCCC(=O)O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('C([C@H](/C=C/C=C/C=C\\\\[C@H](CCCC(O)=O)O)O)/C=C\\\\CCCC(C)O', "
               "'Hydroxy fatty acid with 4 hydroxy group(s)'), "
               "('CCCCC\\\\C=C\\\\C=C\\\\C(O)C\\\\C=C\\\\C=C\\\\C(O)=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('C[C@@H](O)CCCCCCCCCCCCC(O)=O', 'Hydroxy fatty acid with 2 "
               "hydroxy group(s)'), "
               "('OC(CC/C=C\\\\C/C=C\\\\C[C@H](\\\\C=C\\\\C=C\\\\C=C/[C@H](C/C=C\\\\CCO)O)O)=O', "
               "'Hydroxy fatty acid with 4 hydroxy group(s)'), "
               "('C(CC(CCCCCCCCCCC(=O)O)O)(=O)O', 'Hydroxy fatty acid with 3 "
               "hydroxy group(s)'), "
               "('OC(CC/C=C\\\\C/C=C\\\\C/C=C\\\\C=C\\\\[C@H](C/C=C\\\\CCCCC)O)=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('OCCCCCCCCCC(O)CC(O)=O', 'Hydroxy fatty acid with 3 hydroxy "
               "group(s)'), "
               "('OC(C(O)C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)C/C=C\\\\CCCC(O)=O', "
               "'Hydroxy fatty acid with 3 hydroxy group(s)'), "
               "('C(\\\\CC)=C\\\\C/C=C\\\\CC(/C=C/C=C\\\\C/C=C\\\\CCCC(=O)O)O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('C(\\\\C=C/C=C/C=C/[C@H]([C@H](CCCCCO)O)O)=C/[C@H](CCCC(O)=O)O', "
               "'Hydroxy fatty acid with 5 hydroxy group(s)'), "
               "('O[C@@H](CCCC(O)=O)/C=C\\\\C=C\\\\CCC(=O)C/C=C\\\\C=C\\\\[C@@H](O)CC', "
               "'Hydroxy fatty acid with 3 hydroxy group(s)'), "
               "('C(CCC(O)=O)/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCCCCCO', 'Hydroxy "
               "fatty acid with 2 hydroxy group(s)'), "
               "('[H]C(CCCC(O)=O)=CCC([H])=CC([H])=CC(O)CC([H])=CCCCCC', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('CCCCCCCCCCCC(O)CC(O)=O', 'Hydroxy fatty acid with 2 hydroxy "
               "group(s)'), "
               "('C(C(/C=C/C=C/C=C\\\\[C@H](CCCC(O)=O)O)=O)/C=C\\\\CCCCCO', "
               "'Hydroxy fatty acid with 3 hydroxy group(s)'), "
               "('O[C@H](CCCCCCCC)C=CC=CC=C[C@@H](O)CCCC(O)=O', 'Hydroxy fatty "
               "acid with 3 hydroxy group(s)'), "
               "('O=C(CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C(C(C(CCCCC)O)O)O)O', "
               "'Hydroxy fatty acid with 4 hydroxy group(s)'), "
               "('CCCCCCCCCCCCCCCCCCCCCCCC[C@H]([C@H](O)CCCCCCCCCCCCCCCCC[C@H]1C[C@H]1CCCCCCCCCCCCCCCC[C@H](OC)[C@@H](C)CCCCCCCCCCCCCCCCCC)C(O)=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('C(C=CC=C/C=C/[C@H](C/C=C\\\\CC)O)=CC([C@H](CCCCCC(O)=O)O)O', "
               "'Hydroxy fatty acid with 4 hydroxy group(s)'), "
               "('O=C(CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C(/C=C\\\\CCCCC)O)O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('OCCCCCCCCC[C@@H](O)CC(O)=O', 'Hydroxy fatty acid with 3 "
               "hydroxy group(s)'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C=C/C(O)CCCC(O)=O', 'Hydroxy "
               "fatty acid with 2 hydroxy group(s)'), "
               "('C(CCC)C/C=C\\\\C/C=C\\\\C[C@@H]([C@H](C/C=C\\\\CCCC(O)=O)O)O', "
               "'Hydroxy fatty acid with 3 hydroxy group(s)'), "
               "('CCCCC\\\\C=C/C=C/C(O)C\\\\C=C/C\\\\C=C/CCCC(O)=O', 'Hydroxy "
               "fatty acid with 2 hydroxy group(s)'), "
               "('C(C(/C=C/C=C/C=C\\\\[C@H](CCCC(O)=O)O)=O)/C=C\\\\CCCCC=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('O[C@H](CCCC(O)=O)/C=C/C=C\\\\C=C\\\\[C@@H](O)C/C=C\\\\C=C\\\\[C@H](O)CC', "
               "'Hydroxy fatty acid with 4 hydroxy group(s)'), "
               "('C[C@@H](O)CCC\\\\C=C\\\\C=C\\\\C(O)=O', 'Hydroxy fatty acid "
               "with 2 hydroxy group(s)'), ('OC(CCCCCCCCCCC(CCCCCC)O)=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('C(C(C(CCCCCCCCO)O)O)CCCCCCC(=O)O', 'Hydroxy fatty acid with "
               "4 hydroxy group(s)'), "
               "('CCCCCCCCCCCCCCCCCCCCCCCC[C@H]([C@H](O)CCCCCCCCCCCCCCCC[C@@H]1C[C@@H]1[C@H](C)CCCCCCCCCCCCCCCCCCC(=O)C(C)CCCCCCCCCCCCCCCCCC)C(O)=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('C(\\\\CC)=C\\\\C/C=C\\\\C/C=C\\\\CC(/C=C/C=C\\\\CCCC(=O)O)O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('OCCCCCCCCCCCCC\\\\C=C\\\\C(O)=O', 'Hydroxy fatty acid with 2 "
               "hydroxy group(s)')]\n"
               'False positives: '
               "[('[C@@]12([C@H](CCC=C1C=C[C@@H]([C@@H]2CC[C@H](C[C@H](CC(=O)O)O)O)C)OC([C@H](CC)C)=O)[H]', "
               "'Hydroxy fatty acid with 3 hydroxy group(s)'), "
               "('C1=CC(/C(/[C@H]1C/C=C\\\\CCCC([O-])=O)=C/C[C@H](CCCCC)O)=O', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('O[C@H]1[C@@H]([C@@H](CCCCCCCCC(O)=O)C(=O)C1)/C=C/[C@@H](O)CCCCC', "
               "'Hydroxy fatty acid with 3 hydroxy group(s)'), "
               "('OC(C(O)CCCCCCCC(O)=O)CCC(O)CCCCC', 'Hydroxy fatty acid with "
               "4 hydroxy group(s)'), "
               "('OC(=O)\\\\C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('[H]C(CCCC(O)=O)=CCCC([H])=CC([H])=CCC([H])=CCCCCC', 'Hydroxy "
               "fatty acid with 1 hydroxy group(s)'), "
               "('OC1(C2(C(C(CCC2)(C)C)C(O)C=C1CO)C)C(O)=O', 'Hydroxy fatty "
               "acid with 4 hydroxy group(s)'), ('C1=CC(C=C(C1=O)CC(O)=O)=O', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('P(OC[C@H](OC(=O)CCC(O)/C=C/C(O)=O)COC(=O)CCCCCCC/C=C\\\\CCCCCCCC)(OCCN)(O)=O', "
               "'Hydroxy fatty acid with 3 hydroxy group(s)'), "
               "('O=C(CCCC(O)=O)CC', 'Hydroxy fatty acid with 1 hydroxy "
               "group(s)'), "
               "('[C@H]1([C@@H]([C@H](CO[C@H]1C/C(=C/C(OCCCCCCCC([O-])=O)=O)/C)C/C=C/[C@H]([C@H](C)O)C)O)O', "
               "'Hydroxy fatty acid with 3 hydroxy group(s)'), "
               "('CCCCCCC[C@@H](C\\\\C=C\\\\CCC(O)=O)OC', 'Hydroxy fatty acid "
               "with 1 hydroxy group(s)'), "
               "('[O-]C(CCCCCCC/C=C\\\\CCCCCCCCO[C@H]1[C@@H]([C@H]([C@@H]([C@H](O1)CO)O)O)O)=O', "
               "'Hydroxy fatty acid with 4 hydroxy group(s)'), "
               "('O[C@@H](CCCCCCCC)C(O)=O', 'Hydroxy fatty acid with 2 hydroxy "
               "group(s)'), "
               "('OCC=1C(C2(C(C(CCC2)(C)C)CC1)C)CCC(COC(=O)C)=CC(O)=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('O=C1O[C@@H](CCCCCCCCCCCCC[C@@H](O)C)[C@@H](C1=C)C(=O)O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('C(\\\\C=C/CCCCCCCC(=O)[O-])=C/C(C/C=C\\\\CC)OO', 'Hydroxy "
               "fatty acid with 1 hydroxy group(s)'), "
               "('O[C@@H](CCCCCC)CC#CCCCCCCCC(O)=O', 'Hydroxy fatty acid with "
               "2 hydroxy group(s)'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)OC[C@H](COP(O)(=O)OC[C@H](N)C(O)=O)OC(=O)CCCCCCC\\\\C=C/CCCCCC', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('C[C@@H]1O[C@@H](OCCCCCC\\\\C=C\\\\C(O)=O)[C@H](O)C[C@H]1O', "
               "'Hydroxy fatty acid with 3 hydroxy group(s)'), "
               "('O=C(O)CCCCCCCC(O)C(O)CC(O)C(O)CC(O)C(O)C/C=C/CCCCCCC', "
               "'Hydroxy fatty acid with 7 hydroxy group(s)'), "
               "('OC(=O)CCCCCCCCC/C=C\\\\CCCCC', 'Hydroxy fatty acid with 1 "
               "hydroxy group(s)'), ('OC(=O)CCCCCC/C=C\\\\C/C=C\\\\CCCCCC', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('OC(=O)CCCC/C=C\\\\C/C=C\\\\CCCCCC(C)C', 'Hydroxy fatty acid "
               "with 1 hydroxy group(s)'), ('OC(=O)CCCCCCCCCC/C=C\\\\CO', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('C(/C=C\\\\CCCCC=O)\\\\C=C/C=C/C=C/[C@H]([C@@H](O)CCCC(=O)[O-])SC[C@H]([NH3+])C(=O)[O-]', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('C[C@@H]1O[C@@H](OCCCCC[C@@H](O)CC(O)=O)[C@H](O)C[C@H]1O', "
               "'Hydroxy fatty acid with 4 hydroxy group(s)'), "
               "('O([C@@H](C[N+](C)(C)C)CC([O-])=O)C(=O)CCCCCCCCCCCCCC(O)=O', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('C(\\\\[C@H](CCCC([O-])=O)O)=C\\\\C=C\\\\C=C\\\\[C@@H](C\\\\C=C/C=C/C(CC)=O)O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('C[C@H](CCCCC(=O)CC(O)=O)O[C@@H]1O[C@@H](C)[C@H](O)C[C@H]1O', "
               "'Hydroxy fatty acid with 3 hydroxy group(s)'), "
               "('OC(=O)CCC/C=C\\\\C/C=C\\\\C=C\\\\C=O', 'Hydroxy fatty acid "
               "with 1 hydroxy group(s)'), "
               "('O=C1C[C@@](C2=CC=C(C(=O)OCC[C@@](O)(CC(=O)O)C)CC2)(C)C(C1)(C)C', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('O=C(O)C(=CCCCCC)[C@@H](C(=O)O)C', 'Hydroxy fatty acid with 2 "
               "hydroxy group(s)'), ('S1C(N[C@@H](C1)C(O)=O)CCCCCCCCCCC', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('C(=O)(OC[C@H](COP(OC[C@@H](C(=O)[O-])[NH3+])(=O)[O-])OC(=O)CCCCCCCCC(CCCCCCCC)O)CCCCCCCCCCCCCCC', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('CCCCCCCCCCCCCCCCC(=O)OC[C@@H](O)COP([O-])(=O)OC[C@H]([NH3+])C([O-])=O', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('O[C@@H]1C[C@H](O)[C@H](C\\\\C=C/CCCC(O)=O)[C@H]1CCC(=O)CCCCC(O)=O', "
               "'Hydroxy fatty acid with 4 hydroxy group(s)'), "
               "('OC(=O)CCCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC', 'Hydroxy fatty acid "
               "with 1 hydroxy group(s)'), "
               "('O[C@@H]1[C@@H]([C@H](C(C1)=C)/C=C/C(=O)CCCCC)C/C=C\\\\CCCC(O)=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('CC(C(CC)C(O)=O)O', 'Hydroxy fatty acid with 2 hydroxy "
               "group(s)'), ('OC(=O)C(CCCCCCCCCC)C', 'Hydroxy fatty acid with "
               "1 hydroxy group(s)'), "
               "('C(CCCCCC1C(C1)CCCCCCCCCCCCC[C@@H](O)[C@H](C([O-])=O)CCCCCCCCCCCCCCCCCCCCCCCC)CCCCCCCCC2C(C2)CCCCCCCCCCCCCCCCCCCC', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('[H][C@]1(\\\\C=C\\\\[C@@H](O)C(C)CC#CC)[C@H](O)C[C@@H]2C\\\\C(C[C@H]12)=C/CCCC(O)=O', "
               "'Hydroxy fatty acid with 3 hydroxy group(s)'), "
               "('OC(=O)C/C(=C/CCC)/C', 'Hydroxy fatty acid with 1 hydroxy "
               "group(s)'), ('OC(=O)CC(CCCCCC)(C)C', 'Hydroxy fatty acid with "
               "1 hydroxy group(s)'), "
               "('O=C(O)C1[C@@]2(C(C)=C([C@H]1[C@@H](C(C)C)CC2)C=O)C', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('CCCCC\\\\C=C/CCC\\\\C=C\\\\CC\\\\C=C/CCCC(O)=O', 'Hydroxy "
               "fatty acid with 1 hydroxy group(s)'), "
               "('OC(=O)CCC/C=C/C\\\\C=C\\\\C\\\\C=C\\\\CCCCCC', 'Hydroxy "
               "fatty acid with 1 hydroxy group(s)'), "
               "('ClC[C@@]1(O)[C@H]2C(=O)OCC(=C[C@@H]2[C@@H](C(C)C)CC1)C(=O)O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('O=C(O)/C(=C/[C@@H](CCCCC)C)/C', 'Hydroxy fatty acid with 1 "
               "hydroxy group(s)'), ('OC(=O)CCCC#CCC#CCC#CCC#CCCCCCCC', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('O=C(O)/C=C(/C=C/[C@@H]1O[C@]2(O[C@@H]([C@@H](C)CC2)C/C=C(/C=C/[C@H](O)[C@H](/C=C/C(=O)O)C)\\\\C)CC[C@]1(OC(=O)CCC(=O)OC)CCCC)\\\\C', "
               "'Hydroxy fatty acid with 3 hydroxy group(s)'), "
               "('C(=C/C=C/[C@H](CCCCC)OO)/C=C/[C@H](C/C=C\\\\CCCC([O-])=O)OO', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('CCC=CCC=CCC=CCCCCCCCC(=O)O', 'Hydroxy fatty acid with 1 "
               "hydroxy group(s)'), ('OC(=O)C(CCCC(C)C)C', 'Hydroxy fatty acid "
               "with 1 hydroxy group(s)'), ('OC(CCCCCCC)/C=C/CCCCCCCC(O)=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('O(C1C(C(CC1)CC(O)=O)C/C=C/CC)C2OC(C(O)C(O)C2O)CO', 'Hydroxy "
               "fatty acid with 5 hydroxy group(s)'), "
               "('OC(=O)\\\\C=C\\\\CC\\\\C=C\\\\CCC', 'Hydroxy fatty acid with "
               "1 hydroxy group(s)'), "
               "('OC(=O)C1N(CCC1)/C=C/C=C(\\\\O)/C=[N+]\\\\2/C(CCC2)C([O-])=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('CC\\\\C=C/C=C/C=C/CCCCCCCCCC(O)=O', 'Hydroxy fatty acid with "
               "1 hydroxy group(s)'), ('OC(=O)CC(N)CC(C)C', 'Hydroxy fatty "
               "acid with 1 hydroxy group(s)'), ('CCCCCCCC\\\\C=C\\\\CC(O)=O', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCC)CO/C=C\\\\CCCCCCCCCCCCCC)(OC[C@H](N)C(O)=O)(O)=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('CCCCCCCCCCCCCCCCCCC(C)CC(C)\\\\C=C(/C)C(O)=O', 'Hydroxy "
               "fatty acid with 1 hydroxy group(s)'), "
               "('OC(=O)CCC/C=C\\\\CCCCC', 'Hydroxy fatty acid with 1 hydroxy "
               "group(s)'), "
               "('C(C([O-])=O)C/C=C\\\\CC(/C=C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)OO', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('OC(C1CCC(=CC1)C(O)=O)(C)C', 'Hydroxy fatty acid with 2 "
               "hydroxy group(s)'), ('OC(=O)[C@]1(C([C@H](CC1)C(O)=O)(C)C)C', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('CCCCCCCCCCCCCCCC(=O)OC[C@H](COP(O)(=O)OC[C@H](N)C(O)=O)OC(=O)CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CC', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('C(=O)([O-])CC1C(C(CC1)=O)C/C=C\\\\CCO', 'Hydroxy fatty acid "
               "with 1 hydroxy group(s)'), "
               "('OC(=O)CCCCCC#CCC#CCC#CCC#CCC#CCC', 'Hydroxy fatty acid with "
               "1 hydroxy group(s)'), "
               "('[H][C@](COP(OC[C@@](COC(CCCCCCCCCCCCCCCCC)=O)(OC(CCCCCCC/C=C\\\\CCCCCC)=O)[H])(=O)O)(C(O)=O)N', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('OC(=O)C(C(C)C)CCCC', 'Hydroxy fatty acid with 1 hydroxy "
               "group(s)'), "
               "('[H][C@](COP(OC[C@@](COC(CCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CC)=O)(OC(CCCCCCC/C=C\\\\CCCCCCCC)=O)[H])(=O)O)(C(O)=O)N', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('[C@@H]1([C@@H](C[C@H]([C@@H](O1)C)O)O)OCCCCCCCCCCC/C=C/C([O-])=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('OC(=O)CCCCCC/C=C/C=C/C=C/CCCCC', 'Hydroxy fatty acid with 1 "
               "hydroxy group(s)'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCCCC)COC(=O)CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)(OC[C@H](N)C(O)=O)(O)=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('O=C1[C@@H](CCCCC)/C(/C=C1)=C\\\\C=C\\\\C/C=C\\\\CCCC(O)=O', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('O=C1OC(=O)C(=C1CCCCCCCCCCCCCCCC/C=C(\\\\C(=O)O)/CC(=O)O)C', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('CC(C)(C)[C@@H](C(=O)O)NC(=O)OC(C)(C)C', 'Hydroxy fatty acid "
               "with 1 hydroxy group(s)'), ('O=C(O)C[C@H](O)C[C@H](O)CCCCC', "
               "'Hydroxy fatty acid with 3 hydroxy group(s)'), "
               "('C(C(O)=O)C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C[C@@H]1[C@H](CC)O1', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('C([C@H](/C=C/C=C/C=C\\\\[C@H](CCCC([O-])=O)O)O)/C=C\\\\CCCCC([O-])=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('OC1(C(CC(=O)C=C1CO)(C)C)/C=C/C(/C)=C/C(O)=O', 'Hydroxy fatty "
               "acid with 3 hydroxy group(s)'), ('OC(=O)CCCCCCCCCCC(N)CCCCCC', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('OC(=O)\\\\C=C\\\\C=C\\\\CCCCCCCCCCC', 'Hydroxy fatty acid "
               "with 1 hydroxy group(s)'), ('O=C(CCCCCCCCCCCC(O)=O)CCCCC', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('CCCCCCCCCCCCCCSCC(=O)O', 'Hydroxy fatty acid with 1 hydroxy "
               "group(s)'), "
               "('CCCCCC(=O)\\\\C=C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCC(O)=O', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCCCCC)COC(=O)CCCCCCCCC/C=C\\\\CCCCCCCC)(OC[C@H](N)C(O)=O)(O)=O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)'), "
               "('O=C(CCCCCCCCCCCCCCC)C(O)=O', 'Hydroxy fatty acid with 1 "
               "hydroxy group(s)'), "
               "('P(OC[C@H](OC(=O)CCC(=O)/C=C/C(O)=O)COC(=O)CCCCCCCCCCCCCCC)(O)(O)=O', "
               "'Hydroxy fatty acid with 3 hydroxy group(s)'), "
               "('C[C@@H]1OC=C2C(O)=C(C(O)=O)C(=O)C(C)=C2[C@H]1C', 'Hydroxy "
               "fatty acid with 2 hydroxy group(s)'), "
               "('O[C@H]1[C@@H]([C@@H](C(=O)C1)CC)/C=C/[C@@H](O)C/C=C\\\\C/C=C\\\\C/C=C\\\\CCC(O)=O', "
               "'Hydroxy fatty acid with 3 hydroxy group(s)'), "
               "('O=C(O)C(O)CCCCCCCCCCCCCCO[C@@H]1O[C@@H]([C@@H](O[C@@H]2O[C@@H]([C@@H](OC(=O)C)[C@@H]([C@H]2OC(=O)C)OC(=O)C)CO)[C@@H]([C@H]1O)O)COC(=O)C', "
               "'Hydroxy fatty acid with 5 hydroxy group(s)'), "
               "('O[C@@H](CCCCCC)C/C=C\\\\CCCCCCCC(O)=O', 'Hydroxy fatty acid "
               "with 2 hydroxy group(s)'), ('OC(=O)C(CCCC(CCCCCC)C)C', "
               "'Hydroxy fatty acid with 1 hydroxy group(s)'), "
               "('OC(=O)C1=CCNCC1', 'Hydroxy fatty acid with 1 hydroxy "
               "group(s)'), ('OC(=O)CC/C=C\\\\CC=C', 'Hydroxy fatty acid with "
               "1 hydroxy group(s)'), "
               "('C(CCC)C/C=C\\\\C/C=C\\\\C[C@H]([C@@H](C/C=C\\\\CCCC([O-])=O)O)O', "
               "'Hydroxy fatty acid with 2 hydroxy group(s)')]\n"
               "False negatives: [('CC[C@H](O)CC(=O)C(O)=O', 'Too many "
               "heteroatoms for a fatty acid'), ('CC[C@](O)(C(C)=O)C(O)=O', "
               "'Too many heteroatoms for a fatty acid'), "
               "('C[N+](C)(C)C[C@H](O)[C@@H](O)C([O-])=O', 'Too many "
               "heteroatoms for a fatty acid'), ('O=C(O)/C=C(/OC)\\\\CCO', 'No "
               "significant aliphatic chain found'), ('CCC(C)(O)C(O)=O', 'Too "
               "many heteroatoms for a fatty acid'), "
               "('CC[C@](C)(O)C(=O)C(O)=O', 'Too many heteroatoms for a fatty "
               "acid'), "
               "('O=C1C(=O)[C@@H]2[C@@](O)([C@]3(CC[C@@H]4[C@@]([C@H]3C2)(CCCC4(C)C)C)C)C=C1COC(=O)CC(O)(CC(=O)O)C', "
               "'Too many rings for a fatty acid'), ('CC[C@H](O)C(O)=O', 'Too "
               "many heteroatoms for a fatty acid'), ('OC/C(=C/C)/C(O)=O', 'No "
               "significant aliphatic chain found'), "
               "('OC[C@H](O)[C@H](O)C(O)=O', 'Too many heteroatoms for a fatty "
               "acid'), ('OC(CC)C(O)C(O)=O', 'Too many heteroatoms for a fatty "
               "acid'), "
               "('O=C(O)[C@@](O)(CC(=O)NCCCN(O)C(=O)CCCCCCC)CC(=O)NCCCN(O)C(=O)C', "
               "'Contains peptide bond')]",
    'attempt': 4,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 68,
    'num_false_positives': 100,
    'num_true_negatives': 2018,
    'num_false_negatives': 4,
    'num_negatives': None,
    'precision': 0.40476190476190477,
    'recall': 0.9444444444444444,
    'f1': 0.5666666666666667,
    'accuracy': 0.9525114155251142}


"""
Classifies: CHEBI:13601 3-oxo-5alpha-steroid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_3_oxo_5alpha_steroid(smiles: str):
    """
    Determines if a molecule is a 3-oxo-5alpha-steroid.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a 3-oxo-5alpha-steroid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for steroid core (4 fused rings)
    matches = mol.GetSubstructMatches(Chem.MolFromSmarts('[#6]~1~2~[#6]~[#6]~[#6]~3~[#6]~[#6]~[#6]~4~[#6]~[#6]~[#6]~[#6]~4~[#6]~3~[#6]~[#6]~2~[#6]~[#6]~1'))
    
    if not matches:
        return False, "No steroid core structure found"

    # Check for ketone at position 3
    ketone_pattern = Chem.MolFromSmarts('C1CC(=O)CC2')
    if not mol.HasSubstructMatch(ketone_pattern):
        return False, "No ketone group at position 3"

    # Check for alpha configuration at position 5
    # In steroid numbering, position 5 is at the junction of rings A and B
    # Alpha configuration means the hydrogen is pointing down (below the plane)
    
    # First find the position 5 carbon
    steroid_core = matches[0]
    pos5_idx = steroid_core[4]  # Position 5 in the steroid core
    
    # Get the atom and check its stereochemistry
    pos5_atom = mol.GetAtomWithIdx(pos5_idx)
    
    # For 5-alpha steroids, need to check if hydrogen is pointing down (alpha)
    # This can be done by checking if the chiral tag indicates the correct configuration
    chiral_tag = pos5_atom.GetChiralTag()
    
    if chiral_tag == Chem.ChiralType.CHI_UNSPECIFIED:
        return False, "Stereochemistry at position 5 is not specified"
        
    # Create 3D conformation to check stereochemistry
    try:
        mol = Chem.AddHs(mol)
        AllChem.EmbedMolecule(mol, randomSeed=42)
        AllChem.MMFFOptimizeMolecule(mol)
        
        # Get coordinates of relevant atoms
        conf = mol.GetConformer()
        pos5_coords = conf.GetAtomPosition(pos5_idx)
        
        # Check neighbors of pos5 atom to find the hydrogen
        for neighbor in pos5_atom.GetNeighbors():
            if neighbor.GetSymbol() == 'H':
                h_coords = conf.GetAtomPosition(neighbor.GetIdx())
                # Check if H is below the plane (alpha configuration)
                if h_coords.z < pos5_coords.z:
                    return True, "Found 3-oxo-5alpha-steroid structure"
                    
        return False, "Position 5 hydrogen is not in alpha configuration"
        
    except:
        # If 3D embedding fails, fall back to checking just the presence of stereochemistry
        if mol.HasSubstructMatch(Chem.MolFromSmarts('[C@H]1CCC2')):
            return True, "Found 3-oxo-5alpha-steroid structure (based on 2D structure)"
        else:
            return False, "Position 5 hydrogen is not in alpha configuration (based on 2D structure)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:13601',
                          'name': '3-oxo-5alpha-steroid',
                          'definition': 'A 3-oxo steroid that has alpha '
                                        'configuration at position 5.',
                          'parents': ['CHEBI:47788']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183871,
    'num_false_negatives': 6,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999673694915623}


"""
Classifies: CHEBI:24335 glutathione conjugate
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_glutathione_conjugate(smiles: str):
    """
    Determines if a molecule is a glutathione conjugate.

    A glutathione conjugate is defined as any bioconjugate in which glutathione is one of the components.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a glutathione conjugate, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Define the SMARTS pattern for glutathione
    glutathione_pattern = Chem.MolFromSmarts('C(N)C(=O)NCC(=O)O')

    # Check if glutathione is present as a substructure
    match = mol.GetSubstructMatches(glutathione_pattern)
    if match:
        return True, "Molecule is a glutathione conjugate"

    # Check for common glutathione conjugation sites
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'S' and atom.GetDegree() == 2:
            neighbor1 = atom.GetNeighbors()[0]
            neighbor2 = atom.GetNeighbors()[1]
            if neighbor1.GetSymbol() == 'C' and neighbor2.GetSymbol() == 'C':
                if neighbor1.GetDegree() == 3 and neighbor2.GetDegree() == 3:
                    if any(neighbor.GetSmarts() == '[NX3H2]' for neighbor in neighbor1.GetNeighbors()) and \
                       any(neighbor.GetSmarts() == '[CX3](=O)[NX3H2]' for neighbor in neighbor2.GetNeighbors()):
                        return True, "Molecule is a glutathione conjugate"

    return False, "Molecule is not a glutathione conjugate"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24335',
                          'name': 'glutathione conjugate',
                          'definition': 'Any bioconjugate in which glutathione '
                                        'is one of the components',
                          'parents': ['CHEBI:24337', 'CHEBI:64985']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: '
               "[('BrC1=C(O)C(Br)=CC(=C1)/C=C/C=C/C=C/C=C/C=C/C(=O)N[C@@H]2C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(N[C@H](C(O[C@@H]2C)=O)[C@@H](CC)C)=O)CC(=O)N)CC(=O)N)C(C)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCCCN)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](CCC(=O)N)C(O)=O)[C@@H](N)CC=1C=2C(NC1)=CC=CC2', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CCCCN)C(O)=O)[C@@H](NC(=O)[C@@H](N)[C@H](O)C)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(NCC(O)=O)[C@@H](NC(=O)CN)CC(=O)N', 'Molecule is a "
               "glutathione conjugate'), "
               "('BrC1=C(O)C=CC(=C1)/C=C/C=C/C=C/C=C/C(=O)N[C@@H]2C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(N[C@H](C(O[C@@H]2C)=O)C(C)C)=O)CC(=O)N)CC(=O)N)C(C)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CCC(O)=O)C(O)=O)[C@@H](NC(=O)[C@@H](N)C)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C1O[C@@H]([C@H](NC(=O)CC(O)CCCCCCCCCCCCCCN=C(N)N)C(=O)N[C@@H](C(=O)N[C@@H]([C@H](CC)C)C(N[C@@H](C(N[C@@H](C(N[C@@H]1C)=O)CC(=O)N)=O)[C@H](O)C)=O)CC(C)C)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](CC1=CC=CC=C1)C(O)=O)[C@@H](N)C(C)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)NCC(O)=O)[C@@H](N)[C@H](CC)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CO)C(O)=O)[C@@H](NC(=O)CN)CC(=O)N', 'Molecule is "
               "a glutathione conjugate'), "
               "('O=C1OC[C@@H](NC(=O)C[C@H](O)[C@@H](N)CCCCNC(=O)C2=C(O)C(O)=CC=C2)C(=O)N[C@@H](CCCN(O)C=O)C(N[C@@H](C(N[C@H]1CCCN(O)C=O)=O)CC(=O)N)=O', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)NCC(O)=O)[C@@H](N)C(C)C', 'Molecule "
               "is a glutathione conjugate'), "
               "('O=C1O[C@@H]([C@H](NC(=O)CC(O)CCCCCCCCCCCCN=C(N)N)C(=O)N[C@@H](C(=O)N[C@@H]([C@H](CC)C)C(N[C@@H](C(N[C@@H](C(N[C@@H]1C)=O)CC(=O)N)=O)[C@H](O)C)=O)[C@@H](CC)C)C', "
               "'Molecule is a glutathione conjugate'), "
               "('S(CC[C@H](N)C(=O)N[C@@H](CC(=O)N)C(=O)N[C@@H]([C@H](O)C)C(O)=O)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(O)=O)[C@@H](NC(=O)[C@@H](N)CO)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)NCC(O)=O)[C@@H](N)CCC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](CC(O)=O)C(O)=O)[C@@H](N)CCCCN', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC=1C=2C(NC1)=CC=CC2)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCC(=O)N)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('SC[C@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CCCCN)CC(=O)N)C(O)=O', "
               "'Molecule is a glutathione conjugate'), "
               "('C([C@@H](N)CC=1C=CC=CC1)(=O)NCC(=O)NCC(=O)N[C@H](C(=O)N[C@H](C(=O)NCC(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(N[C@H](C(=O)N[C@H](C(N[C@H](C(N[C@H](C(=O)N[C@H](C(=O)N[C@H](C(=O)O)CCC(=O)N)CC(=O)N)C)=O)CC(C)C)=O)CCCCN)CCCNC(=N)N)=O)C)CO)CCCCN)CCCNC(=N)N)C)[C@@H](C)O)CC=2C=CC=CC2', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)NCC(O)=O)[C@@H](N)CC1=CC=C(O)C=C1', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](CC1=CC=CC=C1)C(O)=O)[C@@H](N)CC=2C=3C(NC2)=CC=CC3', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](CC(=O)N)C(O)=O)[C@@H](N)CC=1C=2C(NC1)=CC=CC2', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](CC(=O)N)C(O)=O)[C@@H](N)CCCCN', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC1=CC=C(O)C=C1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(O)=O)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('S(CC[C@H](NC(=O)[C@@H](NC(=O)[C@H]1NCCC1)CC(=O)N)C(O)=O)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](C)C(O)=O)[C@@H](N)CCCN=C(N)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)NCC(O)=O)[C@@H](N)CCCCN', 'Molecule "
               "is a glutathione conjugate'), "
               "('SC[C@H](N)C(=O)N[C@H](C(=O)N[C@@H]([C@H](O)C)C(O)=O)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CCCCN)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCCN=C(N)N)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](CC1=CC=C(O)C=C1)C(O)=O)[C@H]2NCCC2', "
               "'Molecule is a glutathione conjugate'), "
               "('SC[C@H](N)C(=O)N[C@H](C(=O)N[C@@H](CCCCN)C(O)=O)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(C)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC=1C=2C(NC1)=CC=CC2)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CCCN=C(N)N)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCC(O)=O)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(O)=O)[C@@H](NC(=O)CN)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('S(CC[C@H](N)C(=O)N[C@@H](CC(=O)N)C(=O)N[C@@H](CC(=O)N)C(O)=O)C', "
               "'Molecule is a glutathione conjugate'), "
               "('CSCC[C@H](NC(=O)[C@H](CC(C)C)NC(=O)[C@H](Cc1c[nH]c2ccccc12)NC(=O)[C@H](CCC(N)=O)NC(=O)[C@@H](NC(=O)[C@H](Cc1ccccc1)NC(=O)[C@H](CC(O)=O)NC(=O)[C@H](CCC(N)=O)NC(=O)[C@H](C)NC(=O)[C@H](CCCNC(N)=N)NC(=O)[C@H](CCCNC(N)=N)NC(=O)[C@H](CO)NC(=O)[C@H](CC(O)=O)NC(=O)[C@H](CC(C)C)NC(=O)[C@H](Cc1ccc(O)cc1)NC(=O)[C@H](CCCCN)NC(=O)[C@H](CO)NC(=O)[C@H](Cc1ccc(O)cc1)NC(=O)[C@H](CC(O)=O)NC(=O)[C@H](CO)NC(=O)[C@@H](NC(=O)[C@H](Cc1ccccc1)NC(=O)[C@@H](NC(=O)CNC(=O)[C@H](CCC(N)=O)NC(=O)[C@H](CO)NC(=O)[C@@H](N)Cc1cnc[nH]1)[C@@H](C)O)[C@@H](C)O)C(C)C)C(=O)N[C@@H](CC(N)=O)C(=O)N[C@@H]([C@@H](C)O)C(O)=O', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(C)C)C(O)=O)[C@@H](NC(=O)CN)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N1[C@H](CCC1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCCN=C(N)N)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CCCCN)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC1=CC=CC=C1)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](CC1=CC=CC=C1)C(O)=O)[C@@H](N)[C@H](CC)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(C)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC=1NC=NC1)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](CCC(=O)N)C(O)=O)[C@@H](N)CC(C)C', "
               "'Molecule is a glutathione conjugate'), "
               "('SC[C@H](N)C(=O)N[C@H](C(=O)N[C@@H](CCC(O)=O)C(O)=O)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](CO)C(O)=O)[C@@H](N)[C@H](CC)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC1=CC=C(O)C=C1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC2=CC=CC=C2)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](CCC(O)=O)C(O)=O)[C@@H](N)CC(C)C', "
               "'Molecule is a glutathione conjugate'), "
               "('CC(C)[C@H](NC(=O)[C@@H](N)CC(N)=O)C(O)=O', 'Molecule is a "
               "glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(O)=O)[C@@H](NC(=O)[C@@H](N)CO)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('N[C@@H](CC(N)=O)C(=O)N[C@@H](CC(N)=O)C(O)=O', 'Molecule is a "
               "glutathione conjugate'), "
               "('O=C1O[C@@H]([C@H](NC(=O)/C=C/C=C/C=C/C=C/C=C/C2=CC=C(O)C=C2)C(=O)N[C@H](C(=O)N[C@H](C(N[C@H](C(N[C@H]1C(C)C)=O)CC(=O)N)=O)CC(=O)N)C(C)C)C', "
               "'Molecule is a glutathione conjugate'), "
               "('N[C@@H](CC(N)=O)C(=O)N[C@@H](Cc1ccc(O)cc1)C(O)=O', 'Molecule "
               "is a glutathione conjugate'), "
               "('O=C(N[C@@H](CCCCN)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCC(=O)N)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H]([C@H](CC)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC=1C=2C(NC1)=CC=CC2)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@H](C(=O)N[C@@H](C(C)C)C(O)=O)CC(=O)N)[C@@H](N)CC(C)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](C(C)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(O)=O)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('S(CC[C@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CC1=CC=C(O)C=C1)CC(=O)N)C(O)=O)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C1O[C@@H]([C@H](NC(=O)/C=C/C=C/C=C/C=C/C2=CC=C(O)C=C2)C(=O)N[C@H](C(=O)N[C@H](C(N[C@H](C(N[C@H]1CC(C)C)=O)CC(=O)N)=O)CC(=O)N)C(C)C)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)NCC(O)=O)[C@@H](N)CC(C)C', "
               "'Molecule is a glutathione conjugate'), "
               "('BrC1=C(O)C(Br)=CC(=C1)/C=C/C=C/C=C/C=C/C(=O)N[C@H]2C(=O)N[C@@H](C(=O)N[C@H](C(=O)N[C@@H](C(N[C@@H](C(O[C@@H]2C)=O)CC(C)C)=O)CC(=O)N)CC(=O)N)[C@H](CC)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(NCCCC[C@H](N)[C@@H](O)CC(=O)N[C@@H](C(=O)N[C@H](C(=O)N[C@@H](C(=O)N[C@H](C(=O)OC)CCCN(O)C=O)CC(=O)N)CCCN(O)C=O)CO)C1=C(O)C(O)=CC=C1', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC1=CC=CC=C1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC2=CC=C(O)C=C2)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(C)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(=O)N)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O[C@@H]([C@H](N)C(=O)N[C@@H](CC(=O)N)C(=O)N[C@@H](C)C(O)=O)C', "
               "'Molecule is a glutathione conjugate'), "
               "('S(CC[C@H](N)C(=O)N[C@@H](CC(=O)N)C(=O)N[C@@H](CC1=CC=CC=C1)C(O)=O)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N1[C@H](CCC1)C(O)=O)[C@@H](NC(=O)[C@@H](N)[C@H](O)C)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](CCC(=O)N)C(O)=O)[C@H]1NCCC1', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C1O[C@@H]([C@H](NC(=O)CC(O)CCCCCCCCCCCCN=C(N)N)C(=O)N[C@@H](C(=O)N[C@@H](C(C)C)C(N[C@@H](C(N[C@@H](C(N[C@@H]1C)=O)CC(=O)N)=O)[C@H](O)C)=O)[C@H](CC)C)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CCC(=O)N)C(O)=O)[C@@H](NC(=O)[C@@H](N)CO)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](CO)C(O)=O)[C@@H](N)CC1=CC=CC=C1', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(O)=O)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCC(O)=O)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@H](C(=O)N[C@@H]([C@H](CC)C)C(O)=O)CC(=O)N)[C@@H](N)[C@H](CC)C', "
               "'Molecule is a glutathione conjugate'), "
               "('C[C@H](N)C(=O)N[C@@H](Cc1c[nH]c2ccccc12)C(=O)N[C@@H](CC(N)=O)C(=O)N[C@@H](CC(O)=O)C(O)=O', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](C)C(O)=O)[C@@H](N)CC(=O)N', 'Molecule is a "
               "glutathione conjugate'), "
               "('O[C@@H]([C@H](N)C(=O)N[C@@H](CC(=O)N)C(=O)N[C@@H](CO)C(O)=O)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O[C@@H]([C@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CCC(O)=O)CC(=O)N)C(O)=O)C', "
               "'Molecule is a glutathione conjugate'), "
               "('SC[C@H](N)C(=O)N[C@@H](CC(=O)N)C(=O)N[C@@H](C)C(O)=O', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](C)C(O)=O)[C@H]1NCCC1', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CCCN=C(N)N)C(O)=O)[C@@H](NC(=O)CN)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('S(CC[C@H](NC(=O)[C@@H](NC(=O)[C@@H](N)CC(=O)N)CC(=O)N)C(O)=O)C', "
               "'Molecule is a glutathione conjugate'), "
               "('S(CC[C@H](N)C(=O)N[C@@H](CC(=O)N)C(=O)N[C@@H](CCC(=O)N)C(O)=O)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(=O)N)C(=O)N[C@@H](CC(=O)N)C(O)=O)[C@@H](N)CCCN=C(N)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(NCC(O)=O)[C@@H](NC(=O)[C@@H](N)CO)CC(=O)N', 'Molecule is "
               "a glutathione conjugate'), "
               "('SC[C@H](N)C(=O)N[C@H](C(=O)N[C@@H](CC(C)C)C(O)=O)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('S(CCC(NC(=O)C(N)CC(=O)N)C(=O)NC(CC(=O)N)C(=O)NC(CC=1NC=NC1)C(O)=O)C', "
               "'Molecule is a glutathione conjugate'), "
               "('C[C@H](N)C(=O)N[C@@H](CC(N)=O)C(=O)N[C@@H](CC(N)=O)C(=O)N1CCC[C@H]1C(O)=O', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC(O)=O)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(=O)N)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CCC(=O)N)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC1=CC=CC=C1)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](CC1=CC=CC=C1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCC(O)=O)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H]([C@H](CC)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCCN=C(N)N)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C1O[C@@H]([C@H](NC(=O)/C=C/C=C/C=C/C=C/C=C/C2=CC=C(O)C=C2)C(=O)N[C@H](C(=O)N[C@H](C(N[C@H](C(N[C@H]1[C@@H](CC)C)=O)CC(=O)N)=O)CC(=O)N)C(C)C)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](C(C)C)C(=O)N[C@@H](CC(=O)N)C(=O)N[C@@H](CC1=CC=CC=C1)C(O)=O)[C@H]2NCCC2', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C1O[C@@H]([C@H](NC(=O)CC(O)CCCCCCCCCCCCN=C(N)N)C(=O)N[C@@H](C(=O)N[C@@H]([C@H](CC)C)C(N[C@@H](C(N[C@@H](C(N[C@@H]1C)=O)CC(=O)N)=O)C(O)C)=O)C(C)C)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@H](C(=O)N[C@@H](C(C)C)C(O)=O)CC(=O)N)[C@H]1NCCC1', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H]([C@H](CC)C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CCC(O)=O)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('SC[C@H](N)C(=O)N[C@H](C(=O)N[C@@H](C(C)C)C(O)=O)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N1[C@H](CCC1)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(C)C)CC(=O)N', "
               "'Molecule is a glutathione conjugate'), "
               "('S(CC[C@H](N)C(=O)N[C@H](C(=O)N1[C@H](CCC1)C(O)=O)CC(=O)N)C', "
               "'Molecule is a glutathione conjugate'), "
               "('O=C(N[C@@H](C)C(O)=O)[C@@H](NC(=O)[C@@H](N)CC(=O)N)CC(=O)N', "
               "'Molecule is a glutathione conjugate')]\n"
               'False negatives: '
               "[('C(\\\\CC=CC=CC=C[C@@H]([C@@H](C/C=C\\\\CC)O)SC[C@H](NC(CC[C@H](N)C(=O)O)=O)C(=O)NCC(=O)O)=C\\\\C/C=C\\\\CCC(O)=O', "
               "'Molecule is not a glutathione conjugate')]",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 1244,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 1.0,
    'f1': 0.0196078431372549,
    'accuracy': 0.9256505576208178}


"""
Classifies: CHEBI:22527 aminopurine
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdDecomposition

def is_aminopurine(smiles: str):
    """
    Determines if a molecule is an aminopurine (any purine having at least one amino substituent).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an aminopurine, False otherwise
        str: Reason for classification
    """
    
    # Parse SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
    
    # Check for purine scaffold
    purine_smarts = 'c1nc2c([nH]1)ncnc2'
    purine_pattern = Chem.MolFromSmarts(purine_smarts)
    
    if not mol.HasSubstructMatch(purine_pattern):
        return False, "No purine scaffold found"
        
    # Check for amino group attached to purine
    amino_purine_smarts = '[NX3H2]-c1nc2c(n1)ncnc2'
    amino_purine_pattern = Chem.MolFromSmarts(amino_purine_smarts)
    
    if mol.HasSubstructMatch(amino_purine_pattern):
        return True, "Contains purine scaffold with amino substituent"
        
    # Also check for substituted amino groups
    subst_amino_purine_smarts = '[NX3H1;!$(NC=O)]-c1nc2c(n1)ncnc2'
    subst_amino_purine_pattern = Chem.MolFromSmarts(subst_amino_purine_smarts)
    
    if mol.HasSubstructMatch(subst_amino_purine_pattern):
        return True, "Contains purine scaffold with substituted amino group"
        
    return False, "No amino substituent found on purine scaffold"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22527',
                          'name': 'aminopurine',
                          'definition': 'Any purine having at least one amino '
                                        'substituent.',
                          'parents': ['CHEBI:26401']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "cannot import name 'rdDecomposition' from 'rdkit.Chem' "
             '(/Users/cjm/Library/Caches/pypoetry/virtualenvs/c3p-93U7KWO_-py3.11/lib/python3.11/site-packages/rdkit/Chem/__init__.py)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:24589 monosaccharide sulfate
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.rdMolDescriptors import CalcMolFormula

def is_monosaccharide_sulfate(smiles: str):
    """
    Determines if a molecule is a monosaccharide sulfate.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a monosaccharide sulfate, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for presence of sulfate group (SO4)
    matches = mol.GetSubstructMatches(Chem.MolFromSmarts('OS(=O)(=O)[O-,OH]'))
    if not matches:
        return False, "No sulfate group found"

    # Basic monosaccharide pattern - carbon chain with multiple OH groups
    # Look for carbon chain with 5-6 carbons and multiple OH groups
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    oxygen_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'O')
    
    if carbon_count < 5 or carbon_count > 6:
        return False, "Carbon count not consistent with monosaccharide (should be 5-6)"
    
    if oxygen_count < 5:
        return False, "Not enough oxygen atoms for monosaccharide"

    # Check for cyclic structure (pyranose or furanose form)
    ring_info = mol.GetRingInfo()
    if not ring_info.NumRings():
        # Check for open chain form
        aldehyde = mol.HasSubstructMatch(Chem.MolFromSmarts('[CH]=O'))
        ketone = mol.HasSubstructMatch(Chem.MolFromSmarts('C(=O)C'))
        if not (aldehyde or ketone):
            return False, "No ring structure and no aldehyde/ketone group found"
    
    # Check for multiple OH groups
    oh_count = len(mol.GetSubstructMatches(Chem.MolFromSmarts('O[H]')))
    if oh_count < 2:
        return False, "Not enough hydroxyl groups for monosaccharide"

    # If we've made it here, we likely have a monosaccharide sulfate
    sulfate_count = len(matches)
    return True, f"Monosaccharide with {sulfate_count} sulfate group(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24589',
                          'name': 'monosaccharide sulfate',
                          'definition': 'Any carbohydrate sulfate that is a '
                                        'monosaccharide carrying at least one '
                                        'O-sulfo substituent.',
                          'parents': ['CHEBI:35724', 'CHEBI:63367']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183911,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999891252929374}


"""
Classifies: CHEBI:25524 NAD(P)
"""
from rdkit import Chem
from rdkit.Chem import rdMolDescriptors

def is_NAD_P_(smiles: str):
    """
    Determines if a molecule is a NAD(P) coenzyme (NAD or NADP).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a NAD(P) coenzyme, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of the NAD(P) scaffold
    nadp_scaffold = Chem.MolFromSmiles('C1=NC(=NC(=N1)N)N')
    if mol.HasSubstructMatch(nadp_scaffold):
        # Check for the presence of the nicotinamide and ribose moieties
        nicotinamide = Chem.MolFromSmarts('c1ncc(C(=O)N)cn1')
        ribose = Chem.MolFromSmarts('OC(C(O)C(O)C(O)CO)n')
        if mol.HasSubstructMatch(nicotinamide) and mol.HasSubstructMatch(ribose):
            # Check for the presence of phosphate groups
            num_phosphates = len(mol.GetSubstructMatches(Chem.MolFromSmarts('OP(O)(=O)')))
            if num_phosphates == 1:
                return True, "Molecule is NAD"
            elif num_phosphates == 2:
                return True, "Molecule is NADP"
            else:
                return False, "Incorrect number of phosphate groups for NAD(P)"
        else:
            return False, "Missing nicotinamide or ribose moiety"
    else:
        return False, "Missing NAD(P) scaffold"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25524',
                          'name': 'NAD(P)',
                          'definition': 'A coenzyme that may be NAD or NADP.',
                          'parents': ['CHEBI:37007', 'CHEBI:61293']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183920,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0,
    'accuracy': 0.999994562882977}


"""
Classifies: CHEBI:26148 piperidinemonocarboxylic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_piperidinemonocarboxylic_acid(smiles: str):
    """
    Determines if a molecule is a piperidinemonocarboxylic acid (piperidine with one carboxy substituent).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a piperidinemonocarboxylic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for a piperidine ring
    rings = mol.GetRingInfo().AtomRings()
    piperidine_ring = next((ring for ring in rings if len(ring) == 6), None)
    if piperidine_ring is None:
        return False, "No piperidine ring found"

    # Check if the ring contains one nitrogen atom
    ring_atoms = [mol.GetAtomWithIdx(idx) for idx in piperidine_ring]
    nitrogen_count = sum(atom.GetSymbol() == 'N' for atom in ring_atoms)
    if nitrogen_count != 1:
        return False, "Piperidine ring does not contain exactly one nitrogen atom"

    # Check for one carboxy substituent
    carboxy_count = sum(atom.GetSymbol() == 'C' and atom.GetTotalNumHs() == 0 and len(atom.GetNeighbors()) == 3 for atom in mol.GetAtoms())
    if carboxy_count != 1:
        return False, "Molecule does not contain exactly one carboxy group"

    # Check if the carboxy group is attached to the piperidine ring
    carboxy_atom = next(atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'C' and atom.GetTotalNumHs() == 0 and len(atom.GetNeighbors()) == 3)
    is_attached_to_ring = any(neighbor.GetIdx() in piperidine_ring for neighbor in carboxy_atom.GetNeighbors())
    if not is_attached_to_ring:
        return False, "Carboxy group is not attached to the piperidine ring"

    return True, "Molecule is a piperidinemonocarboxylic acid"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26148',
                          'name': 'piperidinemonocarboxylic acid',
                          'definition': 'Any member of the class of '
                                        'piperidines in which one of the '
                                        'carbons of the piperidine ring is '
                                        'substituted by a carboxy group.',
                          'parents': ['CHEBI:25384', 'CHEBI:26151']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 68268,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 1.0,
    'f1': 0.0196078431372549,
    'accuracy': 0.998537348798432}


"""
Classifies: CHEBI:134043 phenylureas
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.MolStandardize import rdMolStandardize

def is_phenylureas(smiles: str):
    """
    Determines if a molecule is a phenylurea (urea with at least one N substituted by phenyl/substituted phenyl).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a phenylurea, False otherwise
        str: Reason for classification
    """
    # Check valid SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Standardize the molecule
    clean_mol = rdMolStandardize.Cleanup(mol)
    
    # SMARTS pattern for urea group with at least one N connected to aromatic ring
    # [NH,NH2]-C(=O)-[NH,NH2] is the urea core
    # One N must be connected to a phenyl ring
    urea_phenyl_pattern = Chem.MolFromSmarts('[cR1]1[cR1][cR1][cR1][cR1][cR1]1-[NH]-C(=O)-[NH][#0,#6]')
    urea_phenyl_pattern2 = Chem.MolFromSmarts('[cR1]1[cR1][cR1][cR1][cR1][cR1]1-[NH]-C(=O)-[N]([#0,#6])[#0,#6]')
    
    # Find matches
    matches1 = clean_mol.GetSubstructMatches(urea_phenyl_pattern)
    matches2 = clean_mol.GetSubstructMatches(urea_phenyl_pattern2)
    
    if not (matches1 or matches2):
        return False, "No phenylurea structure found"
        
    # Get number of phenyl rings attached to urea
    phenyl_count = len(matches1) + len(matches2)
    
    # Check substituents on phenyl rings
    substituents = set()
    for match in matches1 + matches2:
        for atom_idx in match[:6]:  # First 6 atoms are phenyl ring
            atom = clean_mol.GetAtomWithIdx(atom_idx)
            for neighbor in atom.GetNeighbors():
                if neighbor.GetIdx() not in match:
                    substituents.add(neighbor.GetSymbol())
    
    if substituents:
        return True, f"Phenylurea with {phenyl_count} phenyl ring(s) substituted with: {', '.join(sorted(substituents))}"
    else:
        return True, f"Unsubstituted phenylurea with {phenyl_count} phenyl ring(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:134043',
                          'name': 'phenylureas',
                          'definition': 'Any member of the class of ureas in '
                                        'which at least one of the nitrogens '
                                        'of the urea moiety is substituted by '
                                        'a phenyl or substituted phenyl group.',
                          'parents': ['CHEBI:22712', 'CHEBI:47857']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 7,
    'num_false_positives': 100,
    'num_true_negatives': 4515,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.06542056074766354,
    'recall': 1.0,
    'f1': 0.12280701754385964,
    'accuracy': 0.9783643444396365}


"""
Classifies: CHEBI:145039 polyunsaturated fatty ester
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_polyunsaturated_fatty_ester(smiles: str):
    """
    Determines if a molecule is a polyunsaturated fatty ester.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a polyunsaturated fatty ester, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for ester group
    ester_pattern = Chem.MolFromSmarts('[C,c](=O)O[C,c]')
    if not mol.HasSubstructMatch(ester_pattern):
        return False, "No ester group found"
        
    # Count double and triple bonds
    double_bond_pattern = Chem.MolFromSmarts('C=C')
    triple_bond_pattern = Chem.MolFromSmarts('C#C')
    
    double_bonds = len(mol.GetSubstructMatches(double_bond_pattern))
    triple_bonds = len(mol.GetSubstructMatches(triple_bond_pattern))
    
    total_unsaturations = double_bonds + triple_bonds
    
    if total_unsaturations < 2:
        return False, "Not polyunsaturated (less than 2 unsaturations found)"
        
    # Check for fatty acid chain - look for a chain of at least 4 carbons
    fatty_chain_pattern = Chem.MolFromSmarts('CCCC')
    if not mol.HasSubstructMatch(fatty_chain_pattern):
        return False, "No fatty acid chain found (minimum 4 carbons)"
        
    # If we get here, we have:
    # 1. An ester group
    # 2. Multiple unsaturations
    # 3. A fatty acid chain
    reason = f"Found polyunsaturated fatty ester with {double_bonds} double bonds and {triple_bonds} triple bonds"
    return True, reason


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:145039',
                          'name': 'polyunsaturated fatty ester',
                          'definition': 'Any fatty acid ester resulting from '
                                        'the formal esterification of the '
                                        'carboxy group of a polyunsaturated '
                                        'fatty acid.',
                          'parents': ['CHEBI:35748']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 100,
    'num_true_negatives': 1352,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.02912621359223301,
    'recall': 1.0,
    'f1': 0.05660377358490566,
    'accuracy': 0.9312714776632303}


"""
Classifies: CHEBI:33624 bromophenol
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_bromophenol(smiles: str):
    """
    Determines if a molecule is a bromophenol (a phenol containing one or more covalently bonded bromine atoms).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a bromophenol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for phenol substructure
    phenol_pattern = Chem.MolFromSmarts('c1ccc(O)cc1')
    phenol_match = mol.GetSubstructMatches(phenol_pattern)
    if not phenol_match:
        return False, "No phenol substructure found"

    # Check for bromine atoms
    bromine_atoms = [atom.GetIdx() for atom in mol.GetAtoms() if atom.GetAtomicNum() == 35]
    if not bromine_atoms:
        return False, "No bromine atoms found"

    # Check if bromine atoms are covalently bonded to the phenol ring
    for phenol_ring in phenol_match:
        phenol_ring_atoms = set(phenol_ring)
        for bromine_idx in bromine_atoms:
            bromine_neighbors = set([neighbor.GetIdx() for neighbor in mol.GetAtomWithIdx(bromine_idx).GetNeighbors()])
            if bromine_neighbors.intersection(phenol_ring_atoms):
                return True, "Bromophenol found"

    return False, "Bromine atoms not covalently bonded to the phenol ring"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33624',
                          'name': 'bromophenol',
                          'definition': 'A halophenol that is any phenol '
                                        'containing one or more covalently '
                                        'bonded bromine atoms.',
                          'parents': ['CHEBI:37149', 'CHEBI:38856']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: 'int' object is not iterable",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 35702,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 1.0,
    'f1': 0.0196078431372549,
    'accuracy': 0.9972069379660923}


"""
Classifies: CHEBI:23066 cephalosporin
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_cephalosporin(smiles: str):
    """
    Determines if a molecule is a cephalosporin based on its core structure.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a cephalosporin, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Define cephalosporin core SMARTS pattern
    # Core structure is a beta-lactam (4-membered ring) fused to a 6-membered thiazine ring
    ceph_core = Chem.MolFromSmarts('[#6]1[#6][#16][#6][#6]2[#7]1[#6](=O)[#6]2')
    
    if mol.HasSubstructMatch(ceph_core):
        # Check for required carboxylic acid group
        carboxyl = Chem.MolFromSmarts('C(=O)[OH]')
        if not mol.HasSubstructMatch(carboxyl):
            return False, "Missing carboxylic acid group characteristic of cephalosporins"
            
        # Check for characteristic beta-lactam carbonyl
        beta_lactam = Chem.MolFromSmarts('N1C(=O)C2')
        if not mol.HasSubstructMatch(beta_lactam):
            return False, "Missing beta-lactam carbonyl group"
            
        # Additional validation - check for 6-membered thiazine ring
        thiazine = Chem.MolFromSmarts('[#6]1[#6][#16][#6][#6][#7]1')
        if not mol.HasSubstructMatch(thiazine):
            return False, "Missing characteristic 6-membered thiazine ring"
            
        return True, "Contains cephalosporin core structure with required functional groups"
        
    return False, "Does not contain cephalosporin core structure"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23066',
                          'name': 'cephalosporin',
                          'definition': 'A class of beta-lactam antibiotics '
                                        'differing from the penicillins in '
                                        'having a 6-membered, rather than a '
                                        '5-membered, side ring.  Although '
                                        'cephalosporins are among the most '
                                        'commonly used antibiotics in the '
                                        'treatment of routine infections, and '
                                        'their use is increasing over time, '
                                        'they can cause a range of '
                                        'hypersensitivity reactions, from '
                                        'mild, delayed-onset cutaneous '
                                        'reactions to life-threatening '
                                        'anaphylaxis in patients with '
                                        'immunoglobulin E (IgE)-mediated '
                                        'allergy.',
                          'parents': ['CHEBI:38311']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183825,
    'num_false_negatives': 11,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999401640592702}


"""
Classifies: CHEBI:140307 hydroxy fatty acid ascaroside anion
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_hydroxy_fatty_acid_ascaroside_anion(smiles: str):
    """
    Determines if a molecule is a hydroxy fatty acid ascaroside anion.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a hydroxy fatty acid ascaroside anion, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for carboxylate anion
    carboxylate_pattern = Chem.MolFromSmarts('C(=O)[O-]')
    if not mol.HasSubstructMatch(carboxylate_pattern):
        return False, "No carboxylate anion group found"
        
    # Check for ascarylose sugar core
    # Pattern for ascarylose ring with methyl group and hydroxyls
    ascarylose_pattern = Chem.MolFromSmarts('[CH3][CH]1O[CH](O)[CH](O)C[CH]1O')
    if not mol.HasSubstructMatch(ascarylose_pattern):
        return False, "No ascarylose sugar core found"
        
    # Check for linking oxygen between sugar and fatty acid chain
    linker_pattern = Chem.MolFromSmarts('O[CH2]C')
    if not mol.HasSubstructMatch(linker_pattern):
        return False, "No ether linkage found between sugar and fatty acid chain"
        
    # Count carbons in fatty acid chain
    # Get the carboxylate carbon index
    matches = mol.GetSubstructMatches(carboxylate_pattern)
    if len(matches) > 0:
        carboxylate_carbon = matches[0][0]
        
        # Traverse chain from carboxylate to sugar
        visited = set()
        current = carboxylate_carbon
        chain_length = 0
        
        while current is not None:
            visited.add(current)
            next_atom = None
            atom = mol.GetAtomWithIdx(current)
            
            for neighbor in atom.GetNeighbors():
                n_idx = neighbor.GetIdx()
                if n_idx not in visited and neighbor.GetSymbol() == 'C':
                    next_atom = n_idx
                    chain_length += 1
                    break
                    
            current = next_atom
            
        if chain_length < 3:
            return False, "Fatty acid chain too short"
            
    return True, "Contains carboxylate anion, ascarylose core, and appropriate fatty acid chain"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:140307',
                          'name': 'hydroxy fatty acid ascaroside anion',
                          'definition': 'A monocarboxylic acid anion obtained '
                                        'by deprotonation of the carboxy group '
                                        'of any hydroxy fatty acid ascaroside; '
                                        'major species at pH 7.3.',
                          'parents': ['CHEBI:35757']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 100,
    'num_true_negatives': 98128,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.02912621359223301,
    'recall': 1.0,
    'f1': 0.05660377358490566,
    'accuracy': 0.9989819914283679}


"""
Classifies: CHEBI:23639 deoxy sugar
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdDecomposition
from rdkit.Chem.Draw import IPythonConsole
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_deoxy_sugar(smiles: str):
    """
    Determines if a molecule is a deoxy sugar (sugar with a hydroxy group replaced by hydrogen).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a deoxy sugar, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Basic sugar characteristics
    num_carbons = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    num_oxygens = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'O')
    
    # Check if molecule has reasonable number of C and O for a sugar
    if num_carbons < 3 or num_oxygens < 2:
        return False, "Too few carbons or oxygens to be a sugar"
        
    # Check for cyclic structure (furanose or pyranose)
    rings = mol.GetRingInfo()
    if not rings.NumRings():
        # Check for open-chain forms
        if not any(bond.GetBondType() == Chem.BondType.DOUBLE for bond in mol.GetBonds()):
            return False, "Not a cyclic sugar and no carbonyl group found"
            
    # Count hydroxyl groups
    num_hydroxyls = len(Chem.MolFromSmarts('[OH]').GetSubstructMatches(mol))
    
    # For cyclic sugars, typically expect n-1 hydroxyls where n is number of carbons
    # For deoxy sugars, expect even fewer hydroxyls
    if rings.NumRings():
        if num_hydroxyls >= num_carbons:
            return False, "Too many hydroxyl groups to be a deoxy sugar"
            
    # Check for carbonyl groups (aldehydes, ketones)
    has_carbonyl = len(Chem.MolFromSmarts('[C;H1](=O)').GetSubstructMatches(mol)) > 0 or \
                  len(Chem.MolFromSmarts('C(=O)C').GetSubstructMatches(mol)) > 0
                  
    # Look for typical sugar patterns
    sugar_pattern = Chem.MolFromSmarts('[CH2]-[CH1](-[OH])-[CH1](-[OH])')
    deoxy_pattern = Chem.MolFromSmarts('[CH3]-[CH1](-[OH])|[CH2]-[CH1](-[OH])')
    
    if mol.HasSubstructMatch(sugar_pattern) and mol.HasSubstructMatch(deoxy_pattern):
        if rings.NumRings():
            ring_sizes = [len(ring) for ring in rings.AtomRings()]
            if 5 in ring_sizes:
                return True, "Deoxy furanose sugar"
            elif 6 in ring_sizes:
                return True, "Deoxy pyranose sugar"
        elif has_carbonyl:
            return True, "Open-chain deoxy sugar"
            
    if has_carbonyl and mol.HasSubstructMatch(deoxy_pattern):
        return True, "Deoxy sugar derivative"
        
    return False, "Does not match deoxy sugar patterns"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23639',
                          'name': 'deoxy sugar',
                          'definition': 'Any sugar having a hydroxy group '
                                        'replaced with a hydrogen atom.',
                          'parents': ['CHEBI:16646']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "cannot import name 'rdDecomposition' from 'rdkit.Chem' "
             '(/Users/cjm/Library/Caches/pypoetry/virtualenvs/c3p-93U7KWO_-py3.11/lib/python3.11/site-packages/rdkit/Chem/__init__.py)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:25214 metal-sulfur cluster
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_metal_sulfur_cluster(smiles: str):
    """
    Determines if a molecule is a metal-sulfur cluster, defined as a unit with two or more 
    metal atoms and bridging sulfur ligand(s).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a metal-sulfur cluster, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Get all atoms
    atoms = mol.GetAtoms()
    
    # Find metal atoms
    metal_atoms = []
    metal_symbols = ['Sc','Ti','V','Cr','Mn','Fe','Co','Ni','Cu','Zn',
                     'Y','Zr','Nb','Mo','Tc','Ru','Rh','Pd','Ag','Cd',
                     'La','Hf','Ta','W','Re','Os','Ir','Pt','Au','Hg',
                     'Ac','Rf','Db','Sg','Bh','Hs','Mt','Ds','Rg','Cn']
    
    for atom in atoms:
        if atom.GetSymbol() in metal_symbols:
            metal_atoms.append(atom)

    if len(metal_atoms) < 2:
        return False, "Less than two metal atoms found"

    # Find sulfur atoms
    sulfur_atoms = []
    for atom in atoms:
        if atom.GetSymbol() == 'S':
            sulfur_atoms.append(atom)

    if len(sulfur_atoms) == 0:
        return False, "No sulfur atoms found"

    # Check if sulfur atoms are bridging between metals
    bridging_s = False
    for s_atom in sulfur_atoms:
        metal_neighbors = 0
        for neighbor in s_atom.GetNeighbors():
            if neighbor.GetSymbol() in metal_symbols:
                metal_neighbors += 1
        if metal_neighbors >= 2:
            bridging_s = True
            break

    if not bridging_s:
        return False, "No bridging sulfur atoms found between metals"

    # Count number of metals and sulfurs in cluster
    num_metals = len(metal_atoms)
    num_bridging_s = len([s for s in sulfur_atoms if len([n for n in s.GetNeighbors() if n.GetSymbol() in metal_symbols]) >= 2])

    return True, f"Metal-sulfur cluster with {num_metals} metal atoms and {num_bridging_s} bridging sulfur atoms"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25214',
                          'name': 'metal-sulfur cluster',
                          'definition': 'A metal-sulfur cluster is a unit '
                                        'comprising two or more metal atoms '
                                        'and bridging sulfur ligand(s).',
                          'parents': ['CHEBI:33733']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: cannot import name 'rdDecomposition' from "
               "'rdkit.Chem' "
               '(/Users/cjm/Library/Caches/pypoetry/virtualenvs/c3p-93U7KWO_-py3.11/lib/python3.11/site-packages/rdkit/Chem/__init__.py)',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 2,
    'num_true_negatives': 183908,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.5,
    'recall': 1.0,
    'f1': 0.6666666666666666,
    'accuracy': 0.9999891252338075}


"""
Classifies: CHEBI:177332 carbotetracyclic compound
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_carbotetracyclic_compound(smiles: str):
    """
    Determines if a molecule is a carbotetracyclic compound (containing exactly 4 carbocyclic rings).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is carbotetracyclic, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
        
    # Get ring information
    rings = mol.GetRingInfo()
    ring_list = rings.AtomRings()
    
    # Count number of rings
    if len(ring_list) != 4:
        return False, f"Contains {len(ring_list)} rings, not exactly 4 rings"
        
    # Check that all rings are carbocyclic (only carbon atoms)
    for ring in ring_list:
        atoms = [mol.GetAtomWithIdx(i) for i in ring]
        if not all(atom.GetSymbol() == 'C' for atom in atoms):
            return False, "Contains non-carbocyclic ring(s)"
            
    # Check ring sizes
    ring_sizes = [len(ring) for ring in ring_list]
    
    # Success case
    return True, f"Contains 4 carbocyclic rings of sizes: {ring_sizes}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:177332',
                          'name': 'carbotetracyclic compound',
                          'definition': 'A carbopolyclic compound comprising '
                                        'of four carbocyclic rings.',
                          'parents': ['CHEBI:177333', 'CHEBI:35294']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 100,
    'num_true_negatives': 3282,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0196078431372549,
    'recall': 1.0,
    'f1': 0.038461538461538464,
    'accuracy': 0.9704491725768322}